local nkGenie, privateVars = ...

function nkGenie.ExtFrame:new(name, parent, settings)

	local this = {}
	this.name = name
	this.parent = parent
	
	local element = UI.CreateFrame("Frame", name, parent)
	
	if settings ~= nil and settings.color ~= nil and settings.color.border ~= nil then
		element.inner = UI.CreateFrame("nkExtFrame", name .. 'inner', element, { anchors = {{ from = "TOPLEFT", object = element, to = "TOPLEFT", x = 1, y = 1}, { from = "BOTTOMRIGHT", object = element, to = "BOTTOMRIGHT", x = -1, y = -1}}} )
	end
	
	this.element = element
	this.props = {}
	
	setmetatable(this, self)
	self.__index = self
	return this
	
end

function nkGenie.ExtFrame:checkProps(attribute, value) return nkGenie.Tools.checkProps(self, attribute, value) end
function nkGenie.ExtFrame:checkAnchorUpdate(newAnchors) return nkGenie.Tools.checkAnchorUpdate(self, newAnchors) end

function nkGenie.ExtFrame:update (settings)

	if settings == nil then return end
	
	if settings.template ~= nil then
		for k, v in pairs (settings.template) do if settings[k] == nil then settings[k] = v end end
		settings.template = nil
	end
	
	if self:checkProps('width', settings.width) == true then
		self.element:ClearWidth() 
		self.element:SetWidth(settings.width) 
	end
	
	if self:checkProps('height', settings.height) == true then
		self.element:ClearHeight() 
		self.element:SetHeight(settings.height) 
	end
	
	if settings.anchors ~= nil and self:checkAnchorUpdate (settings.anchors) == true then
		for k, anchor in pairs(settings.anchors) do			
			if anchor.x ~= nil and anchor.y ~= nil then				
				self.element:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				self.element:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end
	end
	
	if settings.callBack ~= nil then
		function self.element.Event:LeftClick()
			settings.callBack(self)
		end
	end
	
	if settings.color ~= nil then	
		if settings.color.body ~= nil then
			if settings.color.border ~= nil then
				local alpha = settings.color.alphaborder or 1
				local color = nkGenie.Tools:HexToRGB(settings.color.border)
				self.element:SetBackgroundColor(color[1], color[2], color[3], alpha)
				self.element.inner:update({ color = { body = settings.color.body, alphabody = settings.color.alphabody } })
			else
				local color = nkGenie.Tools:HexToRGB(settings.color.body)
				local alpha = settings.color.alphabody or 1
				self.element:SetBackgroundColor(color[1], color[2], color[3], alpha)
			end
		elseif settings.color.alpha ~= nil then
			self.element:SetAlpha (settings.color.alpha)
		end
	end		
	
	if self:checkProps('layer', settings.layer) == true then self.element:SetLayer(settings.layer) end
	if self:checkProps('strata', settings.strata) == true then self.element:SetStrata(settings.strata) end
	if self:checkProps('visible', settings.visible) == true then self.element:SetVisible(settings.visible) end
	
	for k, v in pairs (settings) do self.props[k] = v end
	
end

function nkGenie.ExtFrame:getElement (innerFlag)

	if (innerFlag == nil or innerFlag == true) and self.element.inner ~= nil then
		return self.element.inner:getElement(false)
	else
		return self.element
	end
		
end

function nkGenie.ExtFrame:SetEvent(event, func)

	local element = self:getElement(false)

	if event == "LeftClick" then
		function element.Event:LeftClick() func(element)  end
	elseif event == "LeftDown" then
		function element.Event:LeftDown() func(element)  end
	elseif event == "MouseMove" then
		function element.Event:MouseMove() func(element)  end
	elseif event == "LeftUp" then
		function element.Event:LeftUp() func(element)  end
	elseif event == "MouseIn" then
		function element.Event:MouseIn() func(element)  end
	elseif event == "MouseOut" then
		function element.Event:MouseOut() func(element)  end
	elseif event == "WheelForward" then
		function element.Event:WheelForward() func(element)  end
	elseif event == "WheelBack" then
		function element.Event:WheelBack() func(element)  end	
	elseif event == "RightClick" then
		function element.Event:RightClick() func(element)  end
	elseif event == "RightDown" then
		function element.Event:RightDown() func(element)  end
	elseif event == "RightUp" then
		function element.Event:RightUp() func(element)  end
	elseif event == "MiddleClick" then
		function element.Event:MiddleClick() func(element)  end
	end
	
end

function nkGenie.ExtFrame:ClearEvent(event)

	local element = self:getElement(false)

	if event == "LeftClick" then
		function element.Event:LeftClick() end
	elseif event == "LeftDown" then
		function element.Event:LeftDown()  end
	elseif event == "MouseMove" then
		function element.Event:MouseMove()  end
	elseif event == "LeftUp" then
		function element.Event:LeftUp() end
	elseif event == "MouseIn" then
		function element.Event:MouseIn() end
	elseif event == "MouseOut" then
		function element.Event:MouseOut() end
	elseif event == "WheelForward" then
		function element.Event:WheelForward() end
	elseif event == "WheelBack" then
		function element.Event:WheelBack() end	
	elseif event == "RightClick" then
		function element.Event:RightClick() end
	elseif event == "RightDown" then
		function element.Event:RightDown() end
	elseif event == "RightUp" then
		function element.Event:RightUp() end
	elseif event == "MiddleClick" then
		function element.Event:MiddleClick() end
	end
	
end

function nkGenie.ExtFrame:SetMacro(event, macro)

	local element = self:getElement(false)

	if event == "LeftClick" then
		element.Event.LeftDown = macro
	elseif event == 'MiddleClick' then
		element.Event.MiddleClick = macro
	end

end

function nkGenie.ExtFrame:ClearAll()

	local height, width = self:GetHeight(), self:GetWidth()
	
	self.element:ClearAll()
	self.element:SetWidth ( width)
	self.element:SetHeight ( height)
	
	self.props.anchors = nil
	self.props.width = width
	self.props.height = height

end

function nkGenie.ExtFrame:ClearAnchors() self:ClearAll() end

function nkGenie.ExtFrame:GetVisible() 

	if self.props.visible ~= nil then self.props.visible = self:getElement(false):GetVisible() end
	return self.props.visible
	
end

function nkGenie.ExtFrame:SetVisible(flag) 
	
	if self:checkProps('visible', flag) == true then 
		self:getElement(false):SetVisible(flag) 
		self.props.visible = flag
	end
	
end

function nkGenie.ExtFrame:SetParent(parent) self:getElement(false):SetParent(parent) end

function nkGenie.ExtFrame:GetWidth() 
	if self.props.width == nil then self.props.width =  self:getElement(false):GetWidth() end
	return self.props.width
end

function nkGenie.ExtFrame:GetHeight() 
	if self.props.height == nil then self.props.height = self:getElement(false):GetHeight() end
	return self.props.height
end

function nkGenie.ExtFrame:SetSecureMode(mode) 
	if self.props.securemode == nil then self.props.securemode = self:getElement(false):SetSecureMode(mode) end
	return self.props.securemode
end

function nkGenie.ExtFrame:GetSecureMode()

	if self.props.securemode == nil then self.props.securemode = self:getElement(false):GetSecureMode() end
	return self.props.securemode
end

function nkGenie.ExtFrame:SetStrata(strata) 
	
	if self:checkProps('strata', strata) == true then 
		self:getElement(false):SetStrata(strata) 
		self.props.strata = strata
	end
end

function nkGenie.ExtFrame:SetLayer(layer) 

	if self:checkProps('layer', layer) == true then 
		self:getElement(false):SetLayer(layer) 
		self.props.layer = layer
	end
end

function nkGenie.ExtFrame:GetAlpha() 
	
	if self.props.color == nil then self.props.color = {} end

	if self.props.color.body ~= nil then
		if self.props.color.border ~= nil then
			if self.props.color.alphaborder == nil then self.props.color.alphaborder = self:getElement(false):GetAlpha() end
			return self.props.color.alphaborder
		else
			if self.props.color.alphabody == nil then self.props.color.alphabody = self:getElement(false):GetAlpha() end
			return self.props.color.alphabody
		end
	else
		if self.props.color.alpha == nil then self.props.color.alpha = self:getElement(false):GetAlpha() end
		return self.props.color.alpha
	end
		
end

function nkGenie.ExtFrame:SetAlpha(value) 
	
	if self.props.color == nil then self.props.color = {} end
	
	if self.props.color.body ~= nil then
		if self.props.color.border ~= nil then
			if self:checkProps('color.alphaborder', value) == true then 
				self:getElement(false):SetAlpha(value) 
				self.props.color.alphaborder = value
			end
		else
			if self:checkProps('color.alphabody', value) == true then 
				self:getElement(false):SetAlpha(value) 
				self.props.color.alphabody = value
			end
		end
	else
		if self:checkProps('color.alpha', value) == true then 
			self:getElement(false):SetAlpha(value) 
			self.props.color.alpha = value
		end
	end
	
end