local nkGenie, privateVars = ...

function nkGenie.ExtTexture:new (name, parent, settings)

	if settings == nil then return nil end
	local this = settings
	this.name = name
	this.parent = parent
	
	local texture = UI.CreateFrame("Texture", name, parent)
	this.element = texture	
	this.props = {}
	
	setmetatable(this, self)
	self.__index = self
	return this

end

function nkGenie.ExtTexture:checkProps(attribute, value) return nkGenie.Tools.checkProps(self, attribute, value) end
function nkGenie.ExtTexture:checkAnchorUpdate(newAnchors) return nkGenie.Tools.checkAnchorUpdate(self, newAnchors) end

function nkGenie.ExtTexture:update (settings) 
	
	if settings == nil then return end
	
	if settings.template ~= nil then
		for k, v in pairs (settings.template) do if settings[k] == nil then settings[k] = v end end
		settings.template = nil
	end
	
	if self:checkProps('width', settings.width) == true then self.element:SetWidth(settings.width) end
	if self:checkProps('height', settings.height) == true then  self.element:SetHeight(settings.height)  end
	
	if settings.anchors ~= nil and self:checkAnchorUpdate (settings.anchors) == true then
		for k, anchor in pairs(settings.anchors) do
			if anchor.x ~= nil and anchor.y ~= nil then				
				self.element:SetPoint (anchor.from, anchor.object, anchor.to, anchor.x, anchor.y)
			else
				self.element:SetPoint (anchor.from, anchor.object, anchor.to)
			end
		end
	end
	
	if self.callBack ~= nil then self:SetEvent('LeftClick', self.callBack) end
	
	if self:checkProps('alpha', settings.alpha) == true then self.element:SetAlpha(settings.alpha) end
	if self:checkProps('layer', settings.layer) == true then self.element:SetLayer(settings.layer) end
	if self:checkProps('visible', settings.visible) == true then self.element:SetVisible (settings.visible) end
	
	if settings.type ~= nil and settings.path ~= nil then self.element:SetTexture(settings.type, settings.path) end	
	
	for k, v in pairs (settings) do self.props[k] = v end
		
end

function nkGenie.ExtTexture:SetTexture(settings)
	
	if settings == nil then return end
	
	local textureType = self.props.type
	if settings.type ~= nil then textureType = settings.type end
	
	self.element:SetTexture(textureType, settings.path)
	self.props.type = textureType
	self.props.path = settings.path
	
end


function nkGenie.ExtTexture:SetEvent(event, func)

	local element = self:getElement(false)

	if event == "LeftClick" then
		function element.Event:LeftClick() func(element)  end
	elseif event == "LeftDown" then
		function element.Event:LeftDown() func(element)  end
	elseif event == "MouseMove" then
		function element.Event:MouseMove() func(element)  end
	elseif event == "LeftUp" then
		function element.Event:LeftUp() func(element)  end
	elseif event == "RightClick" then
		function element.Event:RightClick() func(element)  end
	elseif event == "RightDown" then
		function element.Event:RightDown() func(element)  end
	elseif event == "RightUp" then
		function element.Event:RightUp() func(element)  end
	elseif event == "MiddleClick" then
		function element.Event:MiddleClick() func(element)  end
	elseif event == "MouseIn" then
		function element.Event:MouseIn() func(element)  end
	elseif event == "MouseOut" then
		function element.Event:MouseOut() func(element)  end
	end

end

function nkGenie.ExtTexture:SetMacro(event, macro)

	local element = self:getElement(false)

	if event == "LeftClick" then
		element.Event.LeftDown = macro
	elseif event == 'RightClick' then
		element.Event.RightDown = macro
	end
	
end

function nkGenie.ExtTexture:ClearAll()

	local height, width = self:GetHeight(), self:GetWidth()
	
	self.element:ClearAll()
	self.element:SetWidth ( width)
	self.element:SetHeight ( height)

	self.props.anchors = nil	
	self.props.width = width
	self.props.height = height

end

function nkGenie.ExtTexture:ClearAnchors() self:ClearAll() end

function nkGenie.ExtTexture:getElement() return self.element end

function nkGenie.ExtTexture:GetVisible() 
	if self.props.visible == nil then self.props.visible = self.element:GetVisible() end
	return self.props.visible
end

function nkGenie.ExtTexture:GetHeight()
	
	if self.props.height == nil then self.props.height = self.element:GetHeight() end
	return self.props.height
	
end

function nkGenie.ExtTexture:GetLeft() return self.element:GetLeft() end
function nkGenie.ExtTexture:GetTop() return self.element:GetTop() end

function nkGenie.ExtTexture:GetWidth() 
	
	if self.props.width == nil then self.props.width = self.element:GetWidth() end
	return self.props.width
	
end

function nkGenie.ExtTexture:GetTexture() return self.element:GetTexture() end

function nkGenie.ExtTexture:SetSecureMode(mode) 

	if self:checkProps('securemode', mode) == true then
		self.element:SetSecureMode(mode) 
		self.props.securemode = mode
	end
	
end

function nkGenie.ExtTexture:SetLayer(layer) 
	
	if self:checkProps('layer', layer) == true then
		self.element:SetLayer(layer)
		self.props.layer = layer
	end
	
end

function nkGenie.ExtTexture:SetVisible(flag) 

	if self:checkProps('visible', flag) == true then
		self.element:SetVisible(flag) 
		self.props.visible = flag
	end
end